'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _util = require('util');

var _util2 = _interopRequireDefault(_util);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _nodeUuid = require('node-uuid');

var _nodeUuid2 = _interopRequireDefault(_nodeUuid);

var _request2 = require('request');

var _request3 = _interopRequireDefault(_request2);

var _toughCookieFilestore = require('tough-cookie-filestore');

var _toughCookieFilestore2 = _interopRequireDefault(_toughCookieFilestore);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _logger = require('../lib/logger');

var _logger2 = _interopRequireDefault(_logger);

var _constants = require('./constants');

var constants = _interopRequireWildcard(_constants);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var InstagramSDK = (function () {
  function InstagramSDK(username, password, cookiesFilePath) {
    _classCallCheck(this, InstagramSDK);

    this.instagramHost = 'i.instagram.com';
    this.apiPath = '/api/v1';

    if (!username || !password || !cookiesFilePath) {
      throw new Error('You must specify both `username`, `password` and `cookiesFilePath`.');
    }

    this.isLoggedIn = false;

    this.username = username;
    this.password = password;

    this.cookiesFilePath = cookiesFilePath;

    if (!_fs2.default.existsSync(this.cookiesFilePath)) {
      _fs2.default.writeFileSync(this.cookiesFilePath, '');
    }

    this.jar = _request3.default.jar(new _toughCookieFilestore2.default(this.cookiesFilePath));

    this.uuid = InstagramSDK.generateUUID();
    this.deviceId = InstagramSDK.generateDeviceId();
  }

  //////////////////////////
  //// Helpers
  //////////////////////////

  _createClass(InstagramSDK, [{
    key: 'extractCSRFToken',
    value: function extractCSRFToken(_ref) {
      var res = _ref.res;

      var cookies = this.jar.getCookies(res.request.href),
          CSRFCookie = _lodash2.default.find(cookies, { key: 'csrftoken' });

      if (!CSRFCookie) {
        this.CSRFToken = 'missing';
        return res;
      }

      this.CSRFToken = CSRFCookie.value;
    }

    //////////////////////////
    //// Auth
    //////////////////////////

  }, {
    key: 'login',
    value: function login() {
      var _this = this;

      return this._request({
        path: '/si/fetch_headers/',
        query: {
          challenge_type: 'signup',
          guid: InstagramSDK.generateUUID(false)
        }
      }).tap(this.extractCSRFToken.bind(this)).then(function () {
        var postData = {
          device_id: _this.deviceId,
          guid: _this.uuid,
          username: _this.username,
          password: _this.password,
          csrftoken: _this.CSRFToken,
          login_attempt_count: '0'
        };

        return _this._request({ method: 'POST', path: '/accounts/login/', postData: InstagramSDK.generateSignature(postData) }).tap(_this.extractCSRFToken.bind(_this)).then(InstagramSDK._parseJSON).then(function (data) {
          if (data.status == 'fail') {
            throw new Error(data.message);
          }

          _this.isLoggedIn = true;
          _this.usernameId = data.logged_in_user.pk;
          _this.rankToken = _this.usernameId + '_' + _this.uuid;

          return true;
        });
      });
    }
  }, {
    key: 'checkUsername',
    value: function checkUsername(username) {
      if (!username) {
        throw new Error('Argument `username` is required.');
      }

      /*
       {
         username: 'imposibrus',
         available: false,
         status: 'ok',
         error: 'The username imposibrus is not available.'
       }
      */

      // { username: 'iknergnekjrn', available: true, status: 'ok' }

      return this._request({
        method: 'POST',
        path: '/users/check_username/',
        postData: InstagramSDK.generateSignature({
          _uuid: this.uuid,
          username: username,
          _csrftoken: 'missing'
        })
      }).then(InstagramSDK._parseJSON);
    }

    //////////////////////////
    //// Users
    //////////////////////////

  }, {
    key: 'getSelf',
    value: function getSelf() {
      return this.getUser(this.usernameId);
    }
  }, {
    key: 'getSelfRecentMedia',
    value: function getSelfRecentMedia() {
      var _ref2 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref2$count = _ref2.count;
      var count = _ref2$count === undefined ? 10 : _ref2$count;
      var _ref2$min_id = _ref2.min_id;
      var min_id = _ref2$min_id === undefined ? undefined : _ref2$min_id;
      var _ref2$max_id = _ref2.max_id;
      var max_id = _ref2$max_id === undefined ? undefined : _ref2$max_id;

      return this.getUserRecentMedia(this.usernameId, { count: count, min_id: min_id, max_id: max_id });
    }
  }, {
    key: '_paginate',
    value: function _paginate() {
      var method = arguments.length <= 0 || arguments[0] === undefined ? this.getSelfRecentMedia : arguments[0];

      var _this2 = this;

      var methodArgs = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

      var _ref3 = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      var _ref3$timeout = _ref3.timeout;
      var timeout = _ref3$timeout === undefined ? 0 : _ref3$timeout;
      var _ref3$paginationProp = _ref3.paginationProp;
      var paginationProp = _ref3$paginationProp === undefined ? 'max_id' : _ref3$paginationProp;
      var _ref3$dataProp = _ref3.dataProp;
      var dataProp = _ref3$dataProp === undefined ? 'items' : _ref3$dataProp;

      return new _bluebird2.default(function (resolve, reject) {
        var data = [],
            request = function request(max_id, cb) {
          if (methodArgs.length == 1) {
            methodArgs[0][paginationProp] = max_id;
          } else {
            var lastArg = methodArgs.slice(-1)[0];
            lastArg[paginationProp] = max_id;
            methodArgs.splice(-1, 1, lastArg);
          }

          method.call.apply(method, [_this2].concat(_toConsumableArray(methodArgs))).then(function (resp) {
            if (resp.status != 'ok') {
              return reject('Invalid response: ' + JSON.stringify(resp));
            }

            data = data.concat(resp[dataProp]);
            if (resp.pagination && resp.pagination['next_' + paginationProp]) {
              setTimeout(function () {
                request(resp.pagination['next_' + paginationProp], cb);
              }, timeout);
            } else if (resp['next_' + paginationProp]) {
              setTimeout(function () {
                request(resp['next_' + paginationProp], cb);
              }, timeout);
            } else {
              cb(data);
            }
          }).catch(function (err) {
            _logger2.default.critical('trace', err);
            reject(err);
          });
        };

        request(null, resolve);
      });
    }
  }, {
    key: 'getSelfAllMedia',
    value: function getSelfAllMedia(args) {
      var _this3 = this;

      var _ref4 = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var _ref4$timeout = _ref4.timeout;
      var timeout = _ref4$timeout === undefined ? 0 : _ref4$timeout;

      return new _bluebird2.default(function (resolve, reject) {
        var data = [],
            request = function request(max_id, cb) {
          var _getSelfRecentMedia;

          args[0]['max_id'] = max_id;

          (_getSelfRecentMedia = _this3.getSelfRecentMedia).call.apply(_getSelfRecentMedia, [_this3].concat(_toConsumableArray(args))).then(function (resp) {
            if (resp.status != 'ok') {
              return reject('Invalid response: ' + JSON.stringify(resp));
            }

            data = data.concat(resp.items || []);
            if (resp.items && resp.items.length) {
              setTimeout(function () {
                request(resp.items.slice(-1)[0].id, cb);
              }, timeout);
            } else {
              cb(data);
            }
          }).catch(function (err) {
            _logger2.default.critical('trace', err);
            reject(err);
          });
        };

        request(null, resolve);
      });
    }

    //getSelfRecentLikes({count = 10, max_like_id = undefined} = {}) {
    //  return this._request({path: '/users/self/media/liked', query: {count, max_like_id}}).then(InstagramSDK._parseJSON);
    //}
    //
    //getSelfAllLikes(args, options) {
    //  options.paginationProp = 'max_like_id';
    //  return this._paginate(this.getSelfRecentLikes, args, options);
    //}

  }, {
    key: 'getUser',
    value: function getUser(userID) {
      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      // test/fixtures/userInfo.json
      return this._request({ path: '/users/' + userID + '/info/' }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'getUserRecentMedia',
    value: function getUserRecentMedia(userID) {
      var _ref5 = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var _ref5$count = _ref5.count;
      var count = _ref5$count === undefined ? 10 : _ref5$count;
      var _ref5$min_id = _ref5.min_id;
      var min_id = _ref5$min_id === undefined ? undefined : _ref5$min_id;
      var _ref5$max_id = _ref5.max_id;
      var max_id = _ref5$max_id === undefined ? undefined : _ref5$max_id;

      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      if (!this.isLoggedIn) {
        throw new Error('You must be logged in.');
      }

      // test/fixtures/userFeed.json
      return this._request({ path: '/feed/user/' + userID + '/', query: { count: count, min_id: min_id, max_id: max_id } }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'getUserAllMedia',
    value: function getUserAllMedia(args, options) {
      return this._paginate(this.getUserRecentMedia, args, options);
    }
  }, {
    key: 'usersSearch',
    value: function usersSearch() {
      var _ref6 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref6$q = _ref6.q;
      var q = _ref6$q === undefined ? '' : _ref6$q;
      var _ref6$count = _ref6.count;
      var count = _ref6$count === undefined ? 10 : _ref6$count;

      // test/fixtures/usersSearch.json
      return this._request({ path: '/users/search/', query: { q: q, count: count, rank_token: this.rankToken } }).then(InstagramSDK._parseJSON);
    }

    //////////////////////////
    //// Relationships
    //////////////////////////

    // followed

  }, {
    key: 'getSelfFollows',
    value: function getSelfFollows() {
      var query = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      return this.getUserFollows(this.usernameId, query);
    }
  }, {
    key: 'getSelfAllFollows',
    value: function getSelfAllFollows(args, options) {
      options.paginationProp = 'max_id';

      if (args.length == 0) {
        args.push({});
      }

      return this._paginate(this.getSelfFollows, args, options);
    }
  }, {
    key: 'getUserFollows',
    value: function getUserFollows(userID) {
      var query = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      if (!this.isLoggedIn) {
        throw new Error('You must be logged in.');
      }

      query.rank_token = this.rankToken;

      return this._request({ path: '/friendships/' + userID + '/following/', query: query }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'getUserAllFollows',
    value: function getUserAllFollows(args, options) {
      options.paginationProp = 'max_id';

      if (args.length == 1) {
        args.push({});
      }

      return this._paginate(this.getUserFollows, args, options);
    }

    // followers

  }, {
    key: 'getSelfFollowedBy',
    value: function getSelfFollowedBy() {
      var query = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      return this.getUserFollowers(this.usernameId, query);
    }
  }, {
    key: 'getSelfAllFollowedBy',
    value: function getSelfAllFollowedBy(args, options) {
      options.paginationProp = 'max_id';

      if (args.length == 0) {
        args.push({});
      }

      return this._paginate(this.getSelfFollowedBy, args, options);
    }
  }, {
    key: 'getUserFollowers',
    value: function getUserFollowers(userID) {
      var query = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      query.rank_token = this.rankToken;

      return this._request({ path: '/friendships/' + userID + '/followers/', query: query }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'getUserAllFollowers',
    value: function getUserAllFollowers(args, options) {
      options.paginationProp = 'max_id';

      if (args.length == 1) {
        args.push({});
      }

      return this._paginate(this.getUserFollowers, args, options);
    }

    //getSelfRequestedBy() {
    //  return this._request({path: '/users/self/requested-by'}).then(InstagramSDK._parseJSON);
    //}

  }, {
    key: 'getUserRelationship',
    value: function getUserRelationship(userID) {
      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      // {"status":"ok","incoming_request":false,"outgoing_request":false,"following":false,"followed_by":false,"blocking":false,"is_private":false}
      return this._request({ path: '/friendships/show/' + userID + '/' }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'getUsersRelationships',
    value: function getUsersRelationships(usersIDS) {
      if (!usersIDS || !usersIDS.length) {
        throw new Error('Argument `usersIDS` is required and must be not empty.');
      }

      // test/fixtures/friendshipsMany.json
      return this._request({
        method: 'POST',
        path: '/friendships/show_many/',
        postData: {
          user_ids: usersIDS.join(','),
          _uuid: this.uuid,
          _csrftoken: this.CSRFToken
        }
      }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'followUser',
    value: function followUser(userID) {
      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      if (!this.isLoggedIn) {
        throw new Error('You must be logged in.');
      }

      return this._request({
        method: 'POST',
        path: '/friendships/create/' + userID + '/',
        postData: InstagramSDK.generateSignature({
          _uuid: this.uuid,
          _uid: this.usernameId,
          user_id: userID,
          _csrftoken: this.CSRFToken
        })
      }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'unFollowUser',
    value: function unFollowUser(userID) {
      if (!userID) {
        throw new Error('Argument `userID` is required.');
      }

      if (!this.isLoggedIn) {
        throw new Error('You must be logged in.');
      }

      return this._request({
        method: 'POST',
        path: '/friendships/destroy/' + userID + '/',
        postData: InstagramSDK.generateSignature({
          _uuid: this.uuid,
          _uid: this.usernameId,
          user_id: userID,
          _csrftoken: this.CSRFToken
        })
      }).then(InstagramSDK._parseJSON);
    }

    //////////////////////////
    //// Media
    //////////////////////////

  }, {
    key: 'getMediaInfoById',
    value: function getMediaInfoById(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      if (!this.isLoggedIn) {
        throw new Error('You must be logged in.');
      }

      return this._request({ path: '/media/' + mediaID + '/info/' }).then(InstagramSDK._parseJSON);
    }

    //getMediaInfoByShortCode(shortCode) {
    //  if(!shortCode) {
    //    throw new Error('Argument `shortCode` is required.');
    //  }
    //
    //  return this._request({path: `/media/shortcode/${shortCode}`}).then(InstagramSDK._parseJSON);
    //}
    //
    //mediaSearch({lat = 0, lng = 0, distance = 10} = {}) {
    //  return this._request({path: '/media/search', query: {lat, lng, distance}}).then(InstagramSDK._parseJSON);
    //}

    //////////////////////////
    //// Comments
    //////////////////////////

  }, {
    key: 'getCommentsForMedia',
    value: function getCommentsForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      if (!this.isLoggedIn) {
        throw new Error('You must be logged in.');
      }

      // test/fixtures/mediaComments.json
      return this._request({ path: '/media/' + mediaID + '/comments/' }).then(InstagramSDK._parseJSON);
    }

    //addCommentForMedia(mediaID, text) {
    //  if(!mediaID) {
    //    throw new Error('Argument `mediaID` is required.');
    //  }
    //
    //  if(!text) {
    //    throw new Error('Argument `text` is required.');
    //  }
    //
    //  return this._request({method: 'POST', path: `/media/${mediaID}/comments`, postData: {text}}).then(InstagramSDK._parseJSON);
    //}
    //
    //removeCommentForMedia(mediaID, commentId) {
    //  if(!mediaID) {
    //    throw new Error('Argument `mediaID` is required.');
    //  }
    //
    //  if(!commentId) {
    //    throw new Error('Argument `commentId` is required.');
    //  }
    //
    //  return this._request({method: 'DELETE', path: `/media/${mediaID}/comments/${commentId}`}).then(InstagramSDK._parseJSON);
    //}

    //////////////////////////
    //// Likes
    //////////////////////////

  }, {
    key: 'getLikesForMedia',
    value: function getLikesForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      // test/fixtures/likers.json
      return this._request({ path: '/media/' + mediaID + '/likers/' }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'addLikeForMedia',
    value: function addLikeForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      // {"status":"ok"}
      return this._request({
        method: 'POST',
        path: '/media/' + mediaID + '/like/',
        postData: InstagramSDK.generateSignature({
          _uuid: this.uuid,
          _uid: this.usernameId,
          _csrftoken: this.CSRFToken,
          media_id: mediaID
        })
      }).then(InstagramSDK._parseJSON);
    }
  }, {
    key: 'removeLikeForMedia',
    value: function removeLikeForMedia(mediaID) {
      if (!mediaID) {
        throw new Error('Argument `mediaID` is required.');
      }

      // {"status":"ok"}
      return this._request({
        method: 'POST',
        path: '/media/' + mediaID + '/unlike/',
        postData: InstagramSDK.generateSignature({
          _uuid: this.uuid,
          _uid: this.usernameId,
          _csrftoken: this.CSRFToken,
          media_id: mediaID
        })
      }).then(InstagramSDK._parseJSON);
    }

    //////////////////////////
    //// Tags
    //////////////////////////

    //getTagInfoByTagName(tagName) {
    //  if(!tagName) {
    //    throw new Error('Argument `tagName` is required.');
    //  }
    //
    //  return this._request({path: `/tags/${tagName}`}).then(InstagramSDK._parseJSON);
    //}
    //
    //getRecentMediaForTagName(tagName, {count = 10, min_tag_id = undefined, max_tag_id = undefined} = {}) {
    //  if(!tagName) {
    //    throw new Error('Argument `tagName` is required.');
    //  }
    //
    //  return this._request({path: `/tags/${tagName}/media/recent`, query: {count, min_tag_id, max_tag_id}}).then(InstagramSDK._parseJSON);
    //}
    //
    //getAllMediaForTagName(args, options) {
    //  options.paginationProp = 'max_tag_id';
    //  return this._paginate(this.getRecentMediaForTagName, args, options);
    //}

  }, {
    key: 'tagsSearch',
    value: function tagsSearch() {
      var q = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
      var count = arguments.length <= 1 || arguments[1] === undefined ? 20 : arguments[1];

      return this._request({ path: '/tags/search/', query: { q: q, count: count, rank_token: this.rankToken } }).then(InstagramSDK._parseJSON);
    }

    //////////////////////////
    //// Locations
    //////////////////////////

    //getLocationInfoByLocationId(locationId) {
    //  if(!locationId) {
    //    throw new Error('Argument `locationId` is required.');
    //  }
    //
    //  return this._request({path: `/locations/${locationId}`}).then(InstagramSDK._parseJSON);
    //}
    //
    //getRecentMediaForLocationId(locationId, {count = 10, min_id = undefined, max_id = undefined} = {}) {
    //  if(!locationId) {
    //    throw new Error('Argument `locationId` is required.');
    //  }
    //
    //  return this._request({path: `/locations/${locationId}/media/recent`, query: {count, min_id, max_id}}).then(InstagramSDK._parseJSON);
    //}
    //
    //getAllMediaForLocationId(args, options) {
    //  options.paginationProp = 'max_id';
    //  return this._paginate(this.getRecentMediaForLocationId, args, options);
    //}
    //
    //locationsSearch({
    //    distance = 1000,
    //    facebook_places_id = undefined,
    //    foursquare_id = undefined,
    //    lat = undefined,
    //    lng = undefined,
    //    foursquare_v2_id = undefined
    //    } = {}) {
    //
    //  return this._request({
    //    path: '/locations/search',
    //    query: {
    //      distance,
    //      facebook_places_id,
    //      foursquare_id,
    //      lat,
    //      lng,
    //      foursquare_v2_id
    //    }
    //  }).then(InstagramSDK._parseJSON);
    //}

    //////////////////////////
    //// Embedding
    //////////////////////////

    //getMediaJPGByShortCode(shortCode) {
    //  if(!shortCode) {
    //    throw new Error('Argument `shortCode` is required.');
    //  }
    //
    //  return this._request({path: `/p/${shortCode}/media`}).then(this._getHeaderValue('Location'));
    //}

    //////////////////////////
    //// Subscriptions
    //////////////////////////

    //addSubscription({object = undefined, aspect = undefined, verify_token = undefined, callback_url = undefined} = {}) {
    //  if(!object || !aspect || !callback_url) {
    //    throw new Error('Arguments `object`, `aspect` and `callback_url` is required.');
    //  }
    //
    //  return this._request({
    //    method: 'POST',
    //    path: '/subscriptions',
    //    postData: {
    //      object,
    //      aspect,
    //      verify_token,
    //      callback_url,
    //      client_id: this.clientID,
    //      client_secret: this.clientSecret
    //    }
    //  }).then(InstagramSDK._parseJSON);
    //}
    //
    //getSubscriptions() {
    //  return this._request({
    //    path: '/subscriptions',
    //    query: {
    //      client_id: this.clientID,
    //      client_secret: this.clientSecret
    //    }
    //  }).then(InstagramSDK._parseJSON);
    //}
    //
    //removeSubscription({object = undefined, id = undefined} = {}) {
    //  return this._request({
    //    method: 'DELETE',
    //    path: '/subscriptions',
    //    query: {
    //      client_id: this.clientID,
    //      client_secret: this.clientSecret,
    //      object,
    //      id
    //    }
    //  }).then(InstagramSDK._parseJSON);
    //}

  }, {
    key: '_getHeaderValue',
    value: function _getHeaderValue(headerName) {
      return function () {
        var _ref7 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

        var res = _ref7.res;

        return _bluebird2.default.resolve(res.headers[headerName]);
      };
    }
  }, {
    key: '_request',
    value: function _request() {
      var _this4 = this;

      var _ref8 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _ref8$method = _ref8.method;
      var method = _ref8$method === undefined ? 'GET' : _ref8$method;
      var _ref8$path = _ref8.path;
      var path = _ref8$path === undefined ? '/users/self' : _ref8$path;
      var postData = _ref8.postData;
      var _ref8$query = _ref8.query;
      var query = _ref8$query === undefined ? {} : _ref8$query;

      return new _bluebird2.default(function (resolve, reject) {
        query.access_token = _this4.accessToken;
        var /*contentType = 'application/json',
            */headers = {
          //Accept: contentType,
          'User-Agent': constants.USER_AGENT,
          'X-IG-Connection-Type': 'WIFI',
          'X-IG-Capabilities': 'nQ==',
          'Cookie2': '$Version=1'
        },
            requestOptions = {
          baseUrl: 'https://' + (_this4.instagramHost + _this4.apiPath),
          url: path,
          qs: InstagramSDK._normalizeQueryParams(query),
          method: method,
          jar: _this4.jar
        },
            _postData = undefined;

        if (postData) {
          _postData = JSON.stringify(postData);
          //requestOptions.json = true;
          requestOptions.form = postData;
          //headers['Content-Type'] = 'application/json';
          //headers['Content-length'] = _postData.length;
          _logger2.default.debug('_request: request with postData: %:2j', _postData);
        }

        requestOptions.headers = headers;
        _logger2.default.debug('_request: request with params: %:2j', requestOptions);

        var req = (0, _request3.default)(requestOptions, function (err, res, resData) {
          if (err) {
            return reject(err);
          }

          _logger2.default.debug('_request: response statusCode:', res.statusCode);
          resolve({ res: res, resData: resData });
        });
        console.log('req.headers', req.headers);
      });
    }

    //_oAuth({method = 'POST', path = '/access_token', postData} = {}) {
    //  return new Promise((resolve, reject) => {
    //    let contentType = 'application/json',
    //        headers = {
    //          Accept: contentType
    //        },
    //        requestOptions = {
    //          hostname: this.instagramHost,
    //          path: `/oauth${path}`,
    //          method
    //        },
    //        _postData;
    //
    //    if(postData) {
    //      _postData = querystring.stringify(postData);
    //      headers['Content-Type'] = 'application/x-www-form-urlencoded';
    //      headers['Content-length'] = _postData.length;
    //      logger.debug('_oAuth: request with postData: %:2j', _postData);
    //    }
    //
    //    requestOptions.headers = headers;
    //    logger.debug('_oAuth: request with params: %:2j', requestOptions);
    //
    //    var request = https.request(requestOptions, (res) => {
    //      var resData = '';
    //
    //      res.on('data', (data) => {
    //        resData += data;
    //      });
    //      res.on('end', () => {
    //        logger.debug('_oAuth: response statusCode:', res.statusCode);
    //        resolve({res, resData});
    //      });
    //    }).on('error', reject);
    //
    //    if(postData) {
    //      request.write(_postData);
    //    }
    //
    //    request.end();
    //  });
    //}

  }], [{
    key: 'generateUUID',
    value: function generateUUID() {
      var hyphens = arguments.length <= 0 || arguments[0] === undefined ? true : arguments[0];

      var UUID = _nodeUuid2.default.v4();

      if (!hyphens) {
        return UUID.replace(/-/g, '');
      }

      return UUID;
    }
  }, {
    key: 'generateDeviceId',
    value: function generateDeviceId() {
      return 'android-' + _crypto2.default.randomBytes(8).toString('hex');
    }
  }, {
    key: 'generateSignature',
    value: function generateSignature(data) {
      var json = JSON.stringify(data),
          hash = _crypto2.default.createHmac('SHA256', constants.IG_SIG_KEY).update(json).digest('hex');

      return {
        ig_sig_key_version: constants.SIG_KEY_VERSION,
        signed_body: hash + '.' + json
      };
    }
  }, {
    key: '_parseJSON',
    value: function _parseJSON() {
      var _ref9 = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var res = _ref9.res;
      var resData = _ref9.resData;

      console.log('_parseJSON called');
      var resJSON = {};

      if (res.statusCode == 204) {
        return _bluebird2.default.resolve({});
      }

      try {
        resJSON = JSON.parse(resData);
      } catch (err) {
        _logger2.default.debug('_parseJSON: invalid response:', resData);
        return _bluebird2.default.reject(new Error('Invalid JSON response:' + resData));
      }

      _logger2.default.debug('_parseJSON: response: %.-500s', _util2.default.inspect(resJSON));
      return _bluebird2.default.resolve(resJSON);
    }
  }, {
    key: '_normalizeQueryParams',
    value: function _normalizeQueryParams(query) {
      var out = {};
      for (var i in query) {
        if (query.hasOwnProperty(i)) {
          if (query[i] !== undefined && query[i] !== null) {
            out[i] = query[i];
          }
        }
      }

      return out;
    }
  }]);

  return InstagramSDK;
})();

exports.default = InstagramSDK;